
statvar <- function(I)                                 #calculate the constant in the paper \eta
  {
	temp <- rep(0,1000)
	for (i in 1:1000)
	  {
		epsilon <- matrix(0,nrow=1000,ncol=I)
		for (j in 1:I)
		  {
			epsilon[,j] <- rnorm(1000,0,1)
		  }
		mean <- apply(epsilon,1,mean)
		stat <- apply(abs(epsilon-mean),1,sum)
		temp[i] <- var(stat)
	  }
	return(mean(temp))
	
  }

######################################With-in array estimator of SD, equation (13) in the paper###############

SDinarray <- function(ratios,I)                            #within-array estimate of the standard deviation, s_{W,g}

  {
	n <- nrow(ratios)/I            #pairs that have I replications
      sumsq <- matrix(0,nrow=n,ncol=ncol(ratios))
      temp <- matrix(0,nrow=I,ncol=n)
      withinmean <- matrix(0,nrow=n,ncol=ncol(ratios))   #\bar{Y}_{gj}
      ones <- matrix(1,nrow=I,ncol=n)

      for (i in 1:(ncol(ratios)))  
         {
	      ratio<-ratios[,i]
	      for (j in 1:n)
	         {
		      temp[,j] <- ratio[(I*(j-1)+1):(j*I)]
               }
            mean <- apply(temp,2,mean)
            withinmean[,i] <- mean
            for (j in 1:n)
                {
	             ones[,j] <- mean[j]
                }
            sumsq[,i]<-apply((temp-ones)^2,2,sum)
	    }
	tmp<-apply(sumsq,1,sum)
	SDs<-tmp/(I-1)/ncol(ratios)                     #within-array estimate of the variance, s_{W,g}^2
	
	return(SDs)
	
  }


##########################################REML estimator of SD, equation (14) in the paper###################

SDreml<-function(ratios,I)              
 { 
   nslide <- ncol(ratios)
   n <- nrow(ratios)/I            #pairs that have I replications
   sumsq <- matrix(0,nrow=n,ncol=ncol(ratios))
   temp <- matrix(0,nrow=I,ncol=n)
   withinmean <- matrix(0,nrow=n,ncol=ncol(ratios))   #\bar{Y}_{gj}
   ones <- matrix(1,nrow=I,ncol=n)

   for (i in 1:(ncol(ratios)))  
     {
	   ratio<-ratios[,i]
	   for (j in 1:n)
	     {
		 temp[,j] <- ratio[(I*(j-1)+1):(j*I)]
           }
         mean <- apply(temp,2,mean)
         withinmean[,i] <- mean
         for (j in 1:n)
           {
	       ones[,j] <- mean[j]
           }
         sumsq[,i]<-apply((temp-ones)^2,2,sum)
	}
    tmp<-apply(sumsq,1,sum)
    SDs<-tmp/(I-1)/ncol(ratios)                                     #within-array estimate of the variance, s_{W,g}^2
	  
    allmean <- apply(withinmean,1,mean)
    SDbt <- apply((withinmean-allmean)^2,1,sum)*I/(nslide-1)        #between-array estimate of variance, s_{B,g}^2
   
    rho <- (sum(SDbt)-sum(SDs))/(sum(SDbt)+(I-1)*sum(SDs))                     #estimated correlation
    print(paste("rho",rho))
    sdreml <- ((nslide-1)*SDbt/(1+(I-1)*rho)+nslide*(I-1)*SDs/(1-rho))/(I*nslide-1)   #REML estimate of variance

    return(sdreml)
 }




############################################Validation test#####################################################

validation<-function(ratio,SDs,I)        #the length of ratio SDs is 50; Ratios has the whole length 1716
  {     
	
	ratio<-as.vector(ratio)
	n<-length(ratio)/I
	xi <- sqrt(2*I*(I-1)/pi)
      eta <- statvar(I)                     #the constant in the paper 

	temp <- matrix(0,nrow=n,ncol=I)

	for (j in 1:n)
	   {
		 temp[j,] <- ratio[(I*(j-1)+1):(j*I)]
         }
	mean<-apply(temp,1,mean)
	
	test<-apply((temp-mean)^2,1,sum)
	
	test11<-sum(test)
	test1 <-sum(test/SDs)                #test statitsics T1
	p1<-1-pchisq(test1,df=n*(I-1))       #pvalue for T1
	
      tmp4<-sum(apply(abs(temp-mean),1,sum)/SDs)
	test2<-sqrt(n)*(tmp4/n-xi)/sqrt(eta)                         #test statitsics T2
	p2<-1-pnorm(test2)

	test3<-(sum(test)-(I-1)*sum(SDs))/sqrt(2*(I-1)*sum(SDs^2))    #test statistics T3
	p3 <- 1-pnorm(test3)
	
      test4 <- (sum(abs(temp-mean))- xi*sum(sqrt(SDs)))/sqrt(eta*sum(SDs)) #test statitsics T4
      p4 <- 1-pnorm(test4)
     
      return(p1,p2,p3,p4)
	rm(tmp1,tmp2,tmp3,tmp,tmp4,test22,p1,test1,test11)
  }

